@@ -61,6 +61,20 @@ pub fn matches_simple_var(xs: &str) -> bool {
   true
 }
 
+pub fn matches_digits(xs: &str) -> bool {
+  if xs.is_empty() {
+    return false;
+  }
+  for x in xs.chars() {
+    if is_digit(x) {
+      // ok
+    } else {
+      return false;
+    }
+  }
+  true
+}
+
 pub fn matches_js_var(xs: &str) -> bool {
   if xs.is_empty() {
     false
@@ -105,9 +119,9 @@ pub fn extract_ns_def(s: &str) -> Result<(String, String), String> {
 
 pub fn extract_pkg_from_def(s: &str) -> Option<String> {
   let pieces: Vec<&str> = (&s).split('/').collect();
-  if pieces.len() >= 1 {
+  if !pieces.is_empty() {
     let p2: Vec<&str> = (pieces[0]).split('.').collect();
-    if p2.len() >= 1 {
+    if !p2.is_empty() {
       Some(p2[0].to_string())
     } else {
       None
